function [iPicture, iRed, iGreen, iBlue] = Demon(picture,CFA)
% Read in raw image sensor input and demosaic assuming a BAYER CFA
%Input:
%   picture:        raw image sensor data as uint16 to accept 12-bit ADC
%   CFA:            pattern of the CFA
%                   - 'B1' = Bayer gbrg
%                   - 'B2' = Bayer grbg
%                   - 'B3' = Bayer bggr
%                   - 'B4' = Bayer rggb
%
%Output:
%   iPicture:       array of raw light 0-255 as 3D array full colour image
%   iRed:           just the red channel, 2D array
%   iGreen:         just the green channel, 2D array
%   iBlue:          just the blue channel, 2D array
%
%Future work:
%   Implement additional CFA including Bayer modifications with emerald
%   green and other such non standard CFA arrangements.
%       --> There are several additional non standard bayer patterns
%           --> will be difficult to attend to all

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%   Written By Richard MATTHEWS
%
%   Based on Code originally written 20150317
%   Transferred to function call 20150330
%
%   Mr MATTHEWS is an academic collaborator at:
%   SCHOOL OF ELECTRICAL AND ELECTRONIC ENGINEERING
%   THE UNIVERSITY OF ADELAIDE
%   SA, AUSTRALIA 5005 
%   
%   This work was done as preliminary work for the award of MPhil in part
%   sponsered by the Australian Government Department of Veterans' Affair's
%   via incapacity payments.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

    % initiallise empty arrays for picture demosaicing
 
    [i,j,k] = size(picture);
    
    iRed = zeros(i,j,'int16');
    iGreen = zeros(i,j,'int16');
    iBlue  = zeros(i,j,'int16');
    iPicture  = zeros(i,j,'int16');
    
    switch CFA
        case ' [Green,Blue][Red,Green]' %gbrg
            fprintf('Case not yet implemented');
            % odd row % odd column
            % pixel is green
            iGreen(1:2:end,1:2:end)=picture(1:2:end,1:2:end);
            % odd row % even column
            % pixel is blue
            iBlue(1:2:end,2:2:end)=picture(1:2:end,2:2:end);
            % even row % odd column
            % pixel is red
            iRed(2:2:end,1:2:end)=picture(2:2:end,1:2:end);
            % even row % even column
            % pixel is green
            iGreen(2:2:end,2:2:end)=picture(2:2:end,2:2:end);
        case ' [Green,Red][Blue,Green]' %grbg
            fprintf('Case not yet implemented');
            % odd row % odd column
            % pixel is green
            iGreen(1:2:end,1:2:end)=picture(1:2:end,1:2:end);
            % odd row % even column
            % pixel is red
            iRed(1:2:end,2:2:end)=picture(1:2:end,2:2:end);
            % even row % odd column
            % pixel is blue
            iBlue(2:2:end,1:2:end)=picture(2:2:end,1:2:end);
            % even row % even column
            % pixel is green
            iGreen(2:2:end,2:2:end)=picture(2:2:end,2:2:end);
        case ' [Blue,Green][Green,Red]' %bggr
            fprintf('Case not yet implemented');
            % odd row % odd column
            % pixel is blue
            iBlue(1:2:end,1:2:end)=picture(1:2:end,1:2:end);
            % odd row % even column
            % pixel is green
            iGreen(1:2:end,2:2:end)=picture(1:2:end,2:2:end);
            % even row % odd column
            % pixel is green
            iGreen(2:2:end,1:2:end)=picture(2:2:end,1:2:end);
            % even row % even column
            % pixel is red
            iRed(2:2:end,2:2:end)=picture(2:2:end,2:2:end);
        case ' [Red,Green][Green,Blue]' %rggb
            if(k==1)
            % odd row % odd column
            % pixel is red
            iRed(1:2:end,1:2:end)=picture(1:2:end,1:2:end);
            % odd row % even column
            % pixel is green
            iGreen(1:2:end,2:2:end)=picture(1:2:end,2:2:end);
            % even row % odd column
            % pixel is green
            iGreen(2:2:end,1:2:end)=picture(2:2:end,1:2:end);
            % even row % even column
            % pixel is blue
            iBlue(2:2:end,2:2:end)=picture(2:2:end,2:2:end);  
            else
            % odd row % odd column
            % pixel is red
            iRed(1:2:end,1:2:end)=picture(1:2:end,1:2:end,1);
            % odd row % even column
            % pixel is green
            iGreen(1:2:end,2:2:end)=picture(1:2:end,2:2:end,2);
            % even row % odd column
            % pixel is green
            iGreen(2:2:end,1:2:end)=picture(2:2:end,1:2:end,2);
            % even row % even column
            % pixel is blue
            iBlue(2:2:end,2:2:end)=picture(2:2:end,2:2:end,3);
            end
        case 'empty'
            % odd row % odd column
            % pixel is red
            iRed(1:2:end,1:2:end)=picture(1:2:end,1:2:end);
            % odd row % even column
            % pixel is green
            iGreen(1:2:end,2:2:end)=picture(1:2:end,2:2:end);
            % even row % odd column
            % pixel is green
            iGreen(2:2:end,1:2:end)=picture(2:2:end,1:2:end);
            % even row % even column
            % pixel is blue
            iBlue(2:2:end,2:2:end)=picture(2:2:end,2:2:end);  
    end
   
    iPicture(:,:,1) = iRed;
    iPicture(:,:,2) = iGreen;
    iPicture(:,:,3) = iBlue;
%whos iPicture




