function [PRNU] = getNoiseEstimation(layers)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Image Forensics 
% By Richard MATTHEWS
%
% Created on: 20150317:100900
%
% THE UNIVERSITY OF ADELAIDE
% SA 5005
% AUSTRALIA
%
% Description: This code is written as part of the preliminary work done
% in the first half of 2015 for the award Master of Philosphy.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Focus: Analyse Pentax *ist DL for changes in PRNU under temperature
% differential.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% break image into 512x512 blocks
fprintf('\nAssigning Blocks:....');
smith=[512,512];

% Test image arrays for length issues
[matrix(1,1),matrix(1,2)] = size(layers(:,:,1);
preserve=matrix;
matrix=ceil(matrix./smith);

% pad arrays to exact length
layers = padarray(layers,[matrix(1,1)*smith(1,1)-preserve(1,1) matrix(1,2)*smith(1,2)-preserve(1,2)],'symmetric','post');

neo = zeros(smith(1,1),smith(1,2),matrix(1,1),matrix(1,2));
trinity = neo;
morpheus = neo;



%assign matrixs
for i=1:matrix(1,1);
    fprintf('\n');
    for j=1:matrix(1,2);
        fprintf('X');
        neo(:,:,i,j)=layers((i*smith(1,1)-smith(1,1)+1):i*smith(1,1),(j*smith(1,2)-smith(1,2)+1):j*smith(1,2),1);
        trinity(:,:,i,j)=iRed((i*smith(1,1)-smith(1,1)+1):i*smith(1,1),(j*smith(1,2)-smith(1,2)+1):j*smith(1,2),2);
        morpheus(:,:,i,j)=iRed((i*smith(1,1)-smith(1,1)+1):i*smith(1,1),(j*smith(1,2)-smith(1,2)+1):j*smith(1,2),3);
    end
end
fprintf('\nSuccess');
%% Reduce arrays to just colors with no interpolation spaces
% four options for each location of CFA1 CF2 CF3 and CF4
% for CFA1 array remove even rows and even col
CFA1 = neo(1:2:end,1:2:end,:,:);
% for CFA2 array remove even rows and odd col
CFA2 = trinity(1:2:end,2:2:end,:,:);
% for CFA3 array remove odd rows and even col
CFA3 = trinity(2:2:end,1:2:end,:,:);
% for CFA4 array remove odd rows and odd col
CFA4 = morpheus(2:2:end,2:2:end,:,:);



%% get estimations of noise
fprintf('\nNoise Estimations:....');

patternNoiseCFA1 = CFA1;
patternNoiseCFA2 = CFA2;
patternNoiseCFA3 = CFA3;
patternNoiseCFA4 = CFA4;

for i=1:matrix(1,1);
    fprintf('\n');
    for j=1:matrix(1,2);
        fprintf('X');
        patternNoiseCFA1(:,:,i,j) = getPatternNoise(CFA1(:,:,i,j),3);
        patternNoiseCFA2(:,:,i,j) = getPatternNoise(CFA2(:,:,i,j),3);
        patternNoiseCFA3(:,:,i,j) = getPatternNoise(CFA3(:,:,i,j),3);;
        patternNoiseCFA4(:,:,i,j) = getPatternNoise(CFA4(:,:,i,j),3);
    
    end
end
fprintf('\nSuccess');


%% create sparse noiseless arrays
% from 4 CFA array's create 3 colour specific arrays
patternNoiseRed = neo;
patternNoiseGreen = trinity;
patternNoiseBlue = morpheus;

patternNoiseRed(1:2:end,1:2:end,:,:) = patternNoiseCFA1(:,:,:,:);
patternNoiseGreen(1:2:end,2:2:end,:,:) = patternNoiseCFA2(:,:,:,:);
patternNoiseGreen(2:2:end,1:2:end,:,:) = patternNoiseCFA3(:,:,:,:);
patternNoiseBlue(2:2:end,2:2:end,:,:) = patternNoiseCFA4(:,:,:,:);

%reconstruct matrices
fprintf('\nReconstruct Matrix:....');
neoResult = [];
trinityResult = [];
morpheusResult = [];

for i=1:matrix(1,1);
    fprintf('\n');
    neoTemp = [];
    trinityTemp = [];
    morpheusTemp = [];
    for j=1:matrix(1,2);
        fprintf('X');
        neoTemp = cat(2,neoTemp,patternNoiseRed(:,:,i,j));
        trinityTemp = cat(2,trinityTemp,patternNoiseGreen(:,:,i,j));
        morpheusTemp = cat(2,morpheusTemp,patternNoiseBlue(:,:,i,j));
    end
    neoResult = cat(1,neoResult,neoTemp);
    trinityResult = cat(1,trinityResult,trinityTemp);
    morpheusResult = cat(1,morpheusResult,morpheusTemp);
end

patternNoiseRed = neoResult;
patternNoiseGreen = trinityResult;
patternNoiseBlue = morpheusResult;

% remove padding X

patternNoiseRed(end-(matrix(1,1)*smith(1,1)-preserve(1,1))+1:end,:)=[];
patternNoiseGreen(end-(matrix(1,1)*smith(1,1)-preserve(1,1))+1:end,:)=[];
patternNoiseBlue(end-(matrix(1,1)*smith(1,1)-preserve(1,1))+1:end,:)=[];

%remove padding Y

patternNoiseRed(:,end-(matrix(1,2)*smith(1,2)-preserve(1,2))+1:end)=[];
patternNoiseGreen(:,end-(matrix(1,2)*smith(1,2)-preserve(1,2))+1:end)=[];
patternNoiseBlue(:,end-(matrix(1,2)*smith(1,2)-preserve(1,2))+1:end)=[];

 fprintf('\nSuccess');

%% consturct refrence pattern to parse back

denoisedImage=cat(3,patternNoiseRed,patternNoiseGreen,patternNoiseBlue);

PRNU = double(picture)-denoisedImage; 
 
maximum = max(max(PRNU));
minimum = min(min(PRNU));
average = mean(mean(PRNU));

fprintf('\nThe Maximum pixel value detected is: %f', maximum);
fprintf('\nThe Minimum pixel value detected is: %f', minimum);
fprintf('\nThe Mean pixel value detected is: %f', average);
fprintf('\n'); 

fprintf('\nImage Processed');
fprintf('\n');
