function drawRGB(picture)
% plot in current figure 
%Input:
%   imageFileName:      file name of image
%
%Output:
%   picture:            3040x2024 array of raw light 0-255 
%   bit12:              linear stream of picture data.
%
% EXAMPLE
%
%   myPicture = readPentaxImage('IMGP7741.PEF')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function [picture, bit12] = readPentaxImage(imageFileName)
%
%   Written By Richard MATTHEWS
%
%   Based on Code originally written 20150317
%   Transferred to function call 20150330
%
%   Mr MATTHEWS is an academic collaborator at:
%   SCHOOL OF ELECTRICAL AND ELECTRONIC ENGINEERING
%   THE UNIVERSITY OF ADELAIDE
%   SA, AUSTRALIA 5005 
%   
%   This work was done as preliminary work for the award of MPhil in part
%   sponsered by the Australian Governemtn Department of Veterans' Affairs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% setup
[i,j,~] = size(picture);
%displayPicture = zeros(i,j,3, 'uint8');


%% Convert uint16 to 0-255

% maximum value in 12-bit is 4095
% maximum value in 8-bit is 255
% divide by 4095 multiply by 255 but do this bitwise

picture=double(picture);
%% interpolate
 picture(:,:,1) = interp2(picture(:,:,1),2);
 picture(:,:,2) = interp2(picture(:,:,2),2);
 picture(:,:,3) = interp2(picture(:,:,3),2);

%% scaling
%bit8=picture./(4095);
%bit8=bit8.*255;

%% casting%
%bit8=uint8(bit8);
%picture = bit8;
picture = uint16(picture);

%% Draw RGB Figure
figure();
clf;
subplot(2,2,1);
imshow(picture);

% Red filter
redPicture= picture(1:2:end,1:2:end,1);
subplot(2,2,2);
imshow(redPicture);

% Green filter
greenPicture = picture(1:2:end,2:2:end,2); 
subplot(2,2,3);
imshow(greenPicture);

% Blue Filter
bluePicture = picture(2:2:end,2:2:end,3);
subplot(2,2,4);
imshow(bluePicture);