function [fingerprint] = generateFingerPrint(file,saveFile,crop)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% HELP
% Current method reads in a single image file and produces a ballistic
% fingerprint estimation for that single image only. savefile is used to
% provide a useful description to save the results to a matlab variable
% file for analysis later.
%
%
% LIMITATIONS
% --> Current iteration of method is specifically encoded to work with
%     Pentax *istDL .PEF images only.
%
% FUTURE WORK
% --> provide overloaded methods for folder of images vs single image
% --> Extract Make, Model etc to create a more meaningful savefile
%       --> can do this via EXIF iff EXIF data can be trusted.
% --> Extract image size data from file prior to processing

%% PRELIM

% Image File Types Supported  %% ** ONLY PEF WORKS CURRENTLY ** %%
File_TIFF = '.tiff';
File_TIF = '.tif';
File_PEF = '.PEF';
File_BMP = '.bmp';
File_JPG = '.JPG';
File_jpg = '.jpg';
File_CANNON = '.CR2'; %#ok<NASGU>
noSave = 'noSave';

%% Parse Extension and Match to relevant read method

% get fileName and fileExtension to conduct tests upon
[~,fileName,fileExtension]=fileparts(file);
%fprintf('\nProcessing image: ');
%fprintf(fileName);
%fprintf(fileExtension);
%fprintf('\n');

%% Read in the image
%fprintf('\nImage Read:....');

% Pentax Image format
if strcmp(fileExtension,File_PEF)
    [picture] = readPentaxImage(file);  % 12bit signal
    if crop == 1
        picture = picture(38:1061,38:1061,:);; %crop image to 2048 x 2048 pixels
    end
    picture = int16(picture);
    % picture = double255(picture);% scale to 0-255
  %  fprintf('\t\t\t\t\tPentax');
elseif strcmp(fileExtension,File_TIFF)
    [picture] = readTiffImage(file);  % 12bit signal
    if crop==1
        picture = picture(38:1061,38:1061,:);; %crop image to 2048 x 2048 pixels
    end    
    % picture = double255(picture);% scale to 0-255
   % picture = picture.*64; %shift left 6 %shift from 10 to 16 
    picture = int16(picture);
    % fprintf('\t\t\t\t\tTiff');
elseif strcmp(fileExtension,File_BMP)
    
elseif strcmp(fileExtension,File_JPG)
        [picture] = imread(file); %8-bit depth
        if crop == 1
            picture = picture(38:1061,38:1061,:);; %crop image to 2048 x 2048 pixels
        end
        % picture = double255(picture);% scale to 0-255
        % picture = single(picture);
        %fprintf('\t\t\t\t\tJPEG');
        
elseif strcmp(fileExtension,File_jpg)
        [picture] = imread(file); %8-bit depth
        if crop == 1
            picture = picture(38:1061,38:1061,:);; %crop image to 2048 x 2048 pixels
        end
        % picture = double255(picture);% scale to 0-255
        % picture = single(picture);
        %fprintf('\t\t\t\t\tJPEG');        
else
    fprintf('\nImage type not supported.\n')
    
end

%     %% Print results from image read
%
%     maximum = max(max(picture));
%     minimum = min(min(picture));
%     average = mean(mean(picture));
%
%     fprintf('\nThe Maximum pixel value detected is:        %10.0f', maximum);
%     fprintf('\nThe Minimum pixel value detected is:        %10.0f', minimum);
%     fprintf('\nThe Mean pixel value detected is:           %10.5f', average);
%     fprintf('\n');

%% SCALE IMAGE
%picture = 1.3*picture; %issue with pixels under value 10.
%picture = double(picture);



%% Removal of Saturated pixels

% for a 12-bit signal only pixels in the range of 160-3935 are valid
% these pixels will be replaced with values of 0



%%%
% Redefine hot pixels as top 5% of values and bottom 5% of values
% set these to 0 with a bit mask AFTER zero mean.
%%%
if strcmp(fileExtension,File_PEF)
    picture(picture>(2^12-0.05*2^12))=0; %define hot pixel as one with value 80% or greater.
elseif strcmp(fileExtension,File_TIFF)
    picture(picture>(2^10-0.05*2^10))=0;
    %picture = bitshift(picture,-3); %shift from 10 to 16 
elseif strcmp(fileExtension,File_BMP)
    
elseif strcmp(fileExtension,File_JPG)
    picture(picture>(2^16-0.2*2^16))=0;
    % ignore for now... impossible to ascertain which pixels were saturated.
elseif strcmp(fileExtension,File_jpg)
    %picture(picture>(2^12-0.05*2^12))=0;
    % ignore for now... impossible to ascertain which pixels were saturated.
else
    fprintf('\nImage type not supported.\n')
    
end

%% normalise signal
%    picture = (picture)./(2^16-1);
    
    %% Layerise the image
        %CFA = getCameraCFA(file);
        CFA = ' [Red,Green][Green,Blue]';
        if strcmp(fileExtension,File_PEF)   
           % fprintf('\nDemosaic Image:....');
            [picture, ~, ~, ~] = Demon(picture,CFA);  % 12bit signal
            %fprintf('\t\t\t\tSuccess');
            %picture = unit16(layers);
        elseif strcmp(fileExtension,File_TIF)         
            %fprintf('\nDemosaic Image:....');
            [picture, ~, ~, ~] = Demon(picture,CFA);  % 12bit signal
           % fprintf('\t\t\t\tSuccess');
            %picture = unit16(layers);
        elseif strcmp(fileExtension,File_TIFF)
            %fprintf('\nDemosaic Image:....');
            [picture, ~, ~, ~] = Demon(picture,CFA);  % 12bit signal
            %fprintf('\t\t\t\tSuccess');
            %picture = unit16(layers);
         elseif strcmp(fileExtension,File_BMP)
            
         elseif strcmp(fileExtension,File_JPG)
         elseif strcmp(fileExtension,File_jpg)   
        else
            fprintf('\nImage type not supported.\n')
            
        end
   

%% Get Noise Estimation
PRNU =[];
PRNU = single(PRNU);
PRNU = getNoiseEstimation(picture,CFA);

% L = 4;                                      %  number of decomposition levels
% qmf = MakeONFilter('Daubechies',8);
%        %PRNU(:,:,:,j) = getNoiseEstimation(picture,fileName,CFA);
%        PRNU = single(NoiseExtract(picture,qmf,2,L));
%        %whos PRNU

%% combine layers
%FingerPrint_Grayscale = (0.3.*PRNU(:,:,1)+ 0.6.*PRNU(:,:,2) + 0.1.*PRNU(:,:,1));


%% Residuals
% picture = (picture/1023)*255;
%             % four options for each location of CFA1 CF2 CF3 and CF4
%             % for CFA1 array remove even rows and even col
%             CFA1 = picture(1:2:end,1:2:end,1);
% 
%             % for CFA2 array remove even rows and odd col
%             CFA2 = picture(1:2:end,2:2:end,2);
%  
%             % for CFA3 array remove odd rows and even col
%             CFA3 = picture(2:2:end,1:2:end,2);
% 
%             % for CFA4 array remove odd rows and odd col
%             CFA4 = picture(2:2:end,2:2:end,3);
% blockSize=[256,256]; 
% advance=[128,128];
% 
% % Test image arrays for length issues
% [numberOfCalls(1,1),numberOfCalls(1,2)] = size(CFA1(:,:,1));
% numberOfCalls(1,1)=(numberOfCalls(1,1)./blockSize(1,1))+(numberOfCalls(1,1)./blockSize(1,1)-1);
% numberOfCalls(1,2)=(numberOfCalls(1,2)./blockSize(1,2))+(numberOfCalls(1,2)./blockSize(1,2)-1);
%             
% shiftCFA1    =circshift(CFA1,-0.5.*advance(1,1),1);
% shiftCFA1    =circshift(shiftCFA1,-0.5.*advance(1,2),2);
% 
% shiftCFA2    =circshift(CFA2,-0.5.*advance(1,1),1);
% shiftCFA2    =circshift(shiftCFA2,-0.5.*advance(1,2),2);
% 
% shiftCFA3    =circshift(CFA3,-0.5.*advance(1,1),1);
% shiftCFA3    =circshift(shiftCFA3,-0.5.*advance(1,2),2);
% 
% shiftCFA4    =circshift(CFA4,-0.5.*advance(1,1),1);
% shiftCFA4    =circshift(shiftCFA4,-0.5.*advance(1,2),2);
% 
% 
% % [a,b,c,d] = size(patternNoiseCFA1);
% %remove end elements
% % trimmedCFA1 = zeros(a-advance(1,1),b-advance(1,2),c,d);
% % trimmedCFA2 = trimmedCFA1;
% % trimmedCFA3 = trimmedCFA1;
% % trimmedCFA4 = trimmedCFA1;
% 
% trimmedCFA1 = shiftCFA1(1:1:end-advance(1,1),1:1:end-advance(1,2));
% trimmedCFA2 = shiftCFA2(1:1:end-advance(1,1),1:1:end-advance(1,2));
% trimmedCFA3 = shiftCFA3(1:1:end-advance(1,1),1:1:end-advance(1,2));
% trimmedCFA4 = shiftCFA4(1:1:end-advance(1,1),1:1:end-advance(1,2));
% 
% % Remove Image DC
% 
% DCRemovedCFA1= trimmedCFA1 - mean(mean(trimmedCFA1));
% DCRemovedCFA2= trimmedCFA2 - mean(mean(trimmedCFA2));
% DCRemovedCFA3= trimmedCFA3 - mean(mean(trimmedCFA3));
% DCRemovedCFA4= trimmedCFA4 - mean(mean(trimmedCFA4));
% 
% %FFT method
% % FTCFA1 = fft2(trimmedCFA1);
% % FTCFA1(1,1) = 0;
% % DCRemovedCFA1=ifft2(FTCFA1);
% % 
% % FTCFA2 = fft2(trimmedCFA2);
% % FTCFA2(1,1) = 0;
% % DCRemovedCFA2=ifft2(FTCFA2);
% % %fprintf('\nAssigning Blocks:....');
% 
% %matrix now contains the number of blocks in mxn 
% % FTCFA3 = fft2(trimmedCFA3);
% % FTCFA3(1,1) = 0;
% % DCRemovedCFA3=ifft2(FTCFA3);
% % 
% % FTCFA4 = fft2(trimmedCFA4);
% % FTCFA4(1,1) = 0;
% % DCRemovedCFA4=ifft2(FTCFA4);
% 
% Red = zeros(2.*(numberOfCalls(1,1)*(blockSize(1,1)-advance(1,1))),2.*(numberOfCalls(1,1)*(blockSize(1,1)-advance(1,1))));
% Green = Red;
% Blue = Red;
% 
% % create sparse image in typical file format
% %Red(1:2:end,1:2:end) = DCRemovedCFA1;
% %Green(1:2:end,2:2:end) = DCRemovedCFA2;
% %Green(2:2:end,1:2:end) = DCRemovedCFA3;
% %Blue(2:2:end,2:2:end) = DCRemovedCFA4;
% 
% Red(1:2:end,1:2:end)= trimmedCFA1;
% Green(1:2:end,2:2:end) = trimmedCFA2;
% Green(2:2:end,1:2:end) = trimmedCFA3;
% Blue(2:2:end,2:2:end) = trimmedCFA4;
% % perform k = Y-I
% PICTURE = cat(3,Red,Green,Blue);


%fingerprint = PICTURE - PRNU;
fingerprint = PRNU;
%% publish Refrence Pattern
if strcmp(fileExtension,File_TIFF)
 %   PRNU = bitshift(PRNU,-3); %shift from 10 to 16 
 %   PRNU = bitset(PRNU,16,0,'int16');
 %   PRNU = bitset(PRNU,15,0,'int16');
 %   PRNU = bitset(PRNU,14,0,'int16');
  %  PRNU = bitset(PRNU,13,0,'int16');
  %  PRNU = bitset(PRNU,12,0,'int16');
  %  PRNU = bitset(PRNU,11,0,'int16');
end

%FingerPrint = PRNU; % return full 12-bit fingerprint
%FingerPrint_8bit = FingerPrint./4095.*255; % truncated 8-bit fingerprint

%     maximum = max(max(FingerPrint));
%     minimum = min(min(FingerPrint));
%     average = mean(mean(FingerPrint));
%
%     fprintf('\nThe Maximum pixel value detected is: R:%10.5f G:%10.5f B:%10.5f', maximum(1), maximum(2), maximum(3));
%     fprintf('\nThe Minimum pixel value detected is: R:%10.5f G:%10.5f B:%10.5f', minimum(1), minimum(2), minimum(3));
%     fprintf('\nThe Mean pixel value detected is:    R:%10.5f G:%10.5f B:%10.5f', average(1), average(2), average(3));
%     fprintf('\n');
%% Save Fingerprint
if(strcmp(saveFile,noSave)==0)
    save(saveFile,'FingerPrint');
    %fprintf('\nNoise Approximation Saved.');
    %fprintf('\n');
    
end




%%% FUNCTIONS %%%
function X = double255(X)
%convert to double ranging from 0 to 255
datatype = class(X);
    switch datatype,            % convert to [0,255]
        case 'uint8',  X=double(X);
        case 'uint16', X=ceil(double(X)/4096*255);
    end
