function [PRNU] = getNoiseEstimation(picture,CFA)
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Image Forensics
% By Richard MATTHEWS
%
% Created on: 20150317:100900
%
% THE UNIVERSITY OF ADELAIDE
% SA 5005
% AUSTRALIA
%
% Description: This code is written as part of the preliminary work done
% in the first half of 2015 for the award Master of Philosphy.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%% NEW ALGORITHM 31 MAY 2017
% Read in image 2048 x 2048
% Break down into 4 individual CFA of size 1024x1024
% Remove DC and hot pixels
% Break into smaller images of size 256x256 advancing by overlapping amounts of 128 pixels
% Parse 256x256 images to wavenoise by Goljan => this step can now use any method avaliable
% Crop  pixels around edge of each square
% Merge all squares back together into single CFA
% remove DC 
%
% create sparse image
% for individual CFA now crop outside pixels 
% remove DC
% perform k=Y-I
% Return k

%Main image recompiled = 1792x1792


%% setup

%% MATTHEWS MOD
picture = double(picture); %10bit
picture = (picture/1023)*255; %scale 0-255;

[x,y,z] = size(picture);

tempPic = zeros(x/2,y/2,4) ;

%% Create Sparseless Matrix
switch CFA
        case ' [Green,Blue][Red,Green]' %gbrg
%             % four options for each location of CFA1 CF2 CF3 and CF4
%             % for CFA1 array remove even rows and even col
%             CFA1 = picture(1:2:end,1:2:end,2);
%             tempPic(1:2:end,1:2:end,2) = CFA1;
%             % for CFA2 array remove even rows and odd col
%             CFA2 = picture(1:2:end,2:2:end,3);
%             tempPic(1:2:end,2:2:end,3) = CFA2;
%             % for CFA3 array remove odd rows and even col
%             CFA3 = picture(2:2:end,1:2:end,1);
%             tempPic(2:2:end,1:2:end,1) = CFA3;
%             % for CFA4 array remove odd rows and odd col
%             CFA4 = picture(2:2:end,2:2:end,2);
%             tempPic(2:2:end,2:2:end,2) = CFA4;

            
        case ' [Green,Red][Blue,Green]' %grbg
            % four options for each location of CFA1 CF2 CF3 and CF4
%             % for CFA1 array remove even rows and even col
%             CFA1 = picture(1:2:end,1:2:end,2);
%             tempPic(1:2:end,1:2:end,2) = CFA1;
%             % for CFA2 array remove even rows and odd col
%             CFA2 = picture(1:2:end,2:2:end,1);
%             tempPic(1:2:end,2:2:end,2) = CFA2;
%             % for CFA3 array remove odd rows and even col
%             CFA3 = picture(2:2:end,1:2:end,3);
%             tempPic(2:2:end,1:2:end,3) = CFA3;
%             % for CFA4 array remove odd rows and odd col
%             CFA4 = picture(2:2:end,2:2:end,2);
%             tempPic(2:2:end,2:2:end,2) = CFA4;
        case ' [Blue,Green][Green,Red]' %bggr
            % four options for each location of CFA1 CF2 CF3 and CF4
%             % for CFA1 array remove even rows and even col
%             CFA1 = picture(1:2:end,1:2:end,3);
%             tempPic(1:2:end,1:2:end,3) = CFA1;
%             % for CFA2 array remove even rows and odd col
%             CFA2 = picture(1:2:end,2:2:end,2);
%             tempPic(1:2:end,2:2:end,2) = CFA2;
%             % for CFA3 array remove odd rows and even col
%             CFA3 = picture(2:2:end,1:2:end,2);
%             tempPic(2:2:end,1:2:end,2) = CFA3;
%             % for CFA4 array remove odd rows and odd col
%             CFA4 = picture(2:2:end,2:2:end,1);
%             tempPic(2:2:end,2:2:end,1) = CFA4;
        case ' [Red,Green][Green,Blue]' %rggb
            % four options for each location of CFA1 CF2 CF3 and CF4
            % for CFA1 array remove even rows and even col
            CFA1 = picture(1:2:end,1:2:end,1);
            tempPic(:,:,1) = CFA1;
            % for CFA2 array remove even rows and odd col
            CFA2 = picture(1:2:end,2:2:end,2);
            tempPic(:,:,2) = CFA2;
            % for CFA3 array remove odd rows and even col
            CFA3 = picture(2:2:end,1:2:end,2);
            tempPic(:,:,3) = CFA3;
            % for CFA4 array remove odd rows and odd col
            CFA4 = picture(2:2:end,2:2:end,3);
            tempPic(:,:,4) = CFA4;

        case 'empty'
            % four options for each location of CFA1 CF2 CF3 and CF4
            % for CFA1 array remove even rows and even col
            CFA1 = picture(1:2:end,1:2:end,1);
            tempPic(:,:,1) = CFA1;
            % for CFA2 array remove even rows and odd col
            CFA2 = picture(1:2:end,2:2:end,2);
            tempPic(:,:,2) = CFA2;
            % for CFA3 array remove odd rows and even col
            CFA3 = picture(2:2:end,1:2:end,2);
            tempPic(:,:,3) = CFA3;
            % for CFA4 array remove odd rows and odd col
            CFA4 = picture(2:2:end,2:2:end,3);
            tempPic(:,:,4) = CFA4;
end

%% Remove DC from each of the CFA Arrays

%create concatenation of all four sub arrays
CFA = cat(3,CFA1,CFA2,CFA3,CFA4);

avg = mean(mean(CFA));
disp('Current Average: ');
disp(avg);


% identify top 5% and bottom 5% (saturated and unactivated pixels)
minimums = min(min(CFA));
maximums = max(max(CFA));
range = round(0.05*(maximums-minimums));

bitmaskTop5 = CFA(:,:,:);
for i=1:4
    a=maximums(:,:,i)-range(:,:,i);
    tempArray = bitmaskTop5(:,:,i);
    tempArray(tempArray<(a))=1;
    tempArray(tempArray>(a))=0;
    bitmaskTop5(:,:,i) = tempArray;
    clear tempArray;
end

bitmaskBottom5 = CFA(:,:,:);
for i=1:4
    a=minimums(:,:,i)+range(:,:,i);
    tempArray = bitmaskBottom5(:,:,i);
    tempArray(tempArray<(a))=0;
    tempArray(tempArray>(a))=1;
    bitmaskBottom5(:,:,i) = tempArray;
    clear tempArray;
end

bitmask = and(bitmaskBottom5,bitmaskTop5); % identified pixels are now 0.
bitmaskTop5 = imcomplement(bitmaskTop5); %top 5 are now 1
bitmaskBottom5 = imcomplement(bitmaskBottom5); % bottom 5 are now 1

% Sum all components

top = CFA.*bitmaskTop5;
bottom = CFA.*bitmaskBottom5;

top = sum(sum(top));
bottom = sum(sum(bottom));
S = sum(sum(CFA));
[a,b] = size(CFA);
% S = (S-TOP-BOTTOM)/elements
DC_Offset= (S-top-bottom)/(a*b);
% Perform I = I-S

for i=1:4
CFA(:,:,i) = CFA(:,:,i)-DC_Offset(:,:,i);
end
% set original top 5% and bottom 5% to 0
CFA = CFA.*bitmask;
% check mean
avg = mean(mean(CFA));
disp('DC offset mean Average: ');
disp(avg);

% Zero mean
for i=1:4
CFA(:,:,i) = CFA(:,:,i)-avg(:,:,i);
end

% check mean
avg = mean(mean(CFA));
disp('Zero offset mean Average: ');
disp(avg);

% integrate back to CFA(n)

CFA1 = CFA(:,:,1);
CFA2 = CFA(:,:,2);
CFA3 = CFA(:,:,3);
CFA4 = CFA(:,:,4);



%% check image size;  image size should be 512x512

[a,b,c] = size(CFA1);
if(a==512)
   
elseif(b==512)

else
    disp('error in size of CFA');
end
    

%% break image into 256x256 blocks 

%fprintf('\nAssigning Blocks:....');
blockSize=[256,256]; 
advance=[128,128];

% Test image arrays for length issues
[numberOfCalls(1,1),numberOfCalls(1,2)] = size(tempPic(:,:,1));
numberOfCalls(1,1)=(numberOfCalls(1,1)./blockSize(1,1))+(numberOfCalls(1,1)./blockSize(1,1)-1);
numberOfCalls(1,2)=(numberOfCalls(1,2)./blockSize(1,2))+(numberOfCalls(1,2)./blockSize(1,2)-1);
%matrix now contains the number of blocks in mxn 

% pad arrays to exact length
% no padding is needed as we have cropped to exact size needed for
% algorithm



%% Break into blocks

%initiate arrays
CFA01 = zeros(blockSize(1,1),blockSize(1,2),numberOfCalls(1,1),numberOfCalls(1,2));
CFA02 = CFA01;
CFA03 = CFA01;
CFA04 = CFA01;

for i=1:numberOfCalls(1,1);
    %fprintf('\n');
    for j=1:numberOfCalls(1,2);
        %fprintf('X');
%         
        CFA01(:,:,i,j)        =CFA1(((i-1)*blockSize(1,1)-(i-1)*advance(1,1))+1:(i*blockSize(1,1)-(i-1)*advance(1,1)),(j-1)*blockSize(1,2)-(j-1)*advance(1,2)+1:j*blockSize(1,2)-(j-1)*advance(1,2));   
        CFA02(:,:,i,j)        =CFA2(((i-1)*blockSize(1,1)-(i-1)*advance(1,1))+1:(i*blockSize(1,1)-(i-1)*advance(1,1)),(j-1)*blockSize(1,2)-(j-1)*advance(1,2)+1:j*blockSize(1,2)-(j-1)*advance(1,2));          
        CFA03(:,:,i,j)        =CFA3(((i-1)*blockSize(1,1)-(i-1)*advance(1,1))+1:(i*blockSize(1,1)-(i-1)*advance(1,1)),(j-1)*blockSize(1,2)-(j-1)*advance(1,2)+1:j*blockSize(1,2)-(j-1)*advance(1,2)); 
        CFA04(:,:,i,j)        =CFA4(((i-1)*blockSize(1,1)-(i-1)*advance(1,1))+1:(i*blockSize(1,1)-(i-1)*advance(1,1)),(j-1)*blockSize(1,2)-(j-1)*advance(1,2)+1:j*blockSize(1,2)-(j-1)*advance(1,2)); 
    end
end
%fprintf('\n\t\t\t\t\t\tSuccess');


%% get estimations of noise per layer - Change to per CFA
%fprintf('\nNoise Estimations:....');

patternNoiseCFA1 = CFA01;
patternNoiseCFA2 = CFA02;
patternNoiseCFA3 = CFA03;
patternNoiseCFA4 = CFA04;

L = 4;                                      %  number of decomposition levels
qmf = MakeONFilter('Daubechies',8);  

for i=1:numberOfCalls(1,1);
    %fprintf('\n');
    for j=1:numberOfCalls(1,2);
       % fprintf('X');

        % Richard Code DO NOT USE
%         patternNoiseCFA1(:,:,i,j) = getPatternNoise(CFA1(:,:,i,j),5);
%         patternNoiseCFA2(:,:,i,j) = getPatternNoise(CFA2(:,:,i,j),5);
%         patternNoiseCFA3(:,:,i,j) = getPatternNoise(CFA3(:,:,i,j),5);
%         patternNoiseCFA4(:,:,i,j) = getPatternNoise(CFA4(:,:,i,j),5);

        % Muammar Code - Full Matlab 
%         patternNoiseCFA1(:,:,i,j) =  waveletDenoise(CFA01(:,:,i,j));
%         patternNoiseCFA2(:,:,i,j) =  waveletDenoise(CFA02(:,:,i,j));
%         patternNoiseCFA3(:,:,i,j) =  waveletDenoise(CFA03(:,:,i,j));
%         patternNoiseCFA4(:,:,i,j) =  waveletDenoise(CFA04(:,:,i,j));
     
        % Goljan Code - Implements C Libraries - May not be Phoenix compatible
        patternNoiseCFA1(:,:,i,j) = single(NoiseExtract(CFA01(:,:,i,j),qmf,5,L));
        patternNoiseCFA2(:,:,i,j) = single(NoiseExtract(CFA02(:,:,i,j),qmf,5,L));
        patternNoiseCFA3(:,:,i,j) = single(NoiseExtract(CFA03(:,:,i,j),qmf,5,L));
        patternNoiseCFA4(:,:,i,j) = single(NoiseExtract(CFA04(:,:,i,j),qmf,5,L));
    end
end
%fprintf('\n\t\t\t\t\t\tSuccess');



%% Crop outside pixels due to Edge Effects

%shift elements to be cropped to outer edges
patternNoiseCFA1    =circshift(patternNoiseCFA1,-0.5.*advance(1,1),1);
patternNoiseCFA1    =circshift(patternNoiseCFA1,-0.5.*advance(1,2),2);

patternNoiseCFA2    =circshift(patternNoiseCFA2,-0.5.*advance(1,1),1);
patternNoiseCFA2    =circshift(patternNoiseCFA2,-0.5.*advance(1,2),2);

patternNoiseCFA3    =circshift(patternNoiseCFA3,-0.5.*advance(1,1),1);
patternNoiseCFA3    =circshift(patternNoiseCFA3,-0.5.*advance(1,2),2);

patternNoiseCFA4    =circshift(patternNoiseCFA4,-0.5.*advance(1,1),1);
patternNoiseCFA4    =circshift(patternNoiseCFA4,-0.5.*advance(1,2),2);


% [a,b,c,d] = size(patternNoiseCFA1);
%remove end elements
% trimmedCFA1 = zeros(a-advance(1,1),b-advance(1,2),c,d);
% trimmedCFA2 = trimmedCFA1;
% trimmedCFA3 = trimmedCFA1;
% trimmedCFA4 = trimmedCFA1;

trimmedCFA1 = patternNoiseCFA1(1:1:end-advance(1,1),1:1:end-advance(1,2),:,:); 
trimmedCFA2 = patternNoiseCFA2(1:1:end-advance(1,1),1:1:end-advance(1,2),:,:);
trimmedCFA3 = patternNoiseCFA3(1:1:end-advance(1,1),1:1:end-advance(1,2),:,:);
trimmedCFA4 = patternNoiseCFA4(1:1:end-advance(1,1),1:1:end-advance(1,2),:,:);



%% merge blocks to individual CFA

mergedCFA1 = [];
mergedCFA2 = mergedCFA1;
mergedCFA3 = mergedCFA1;
mergedCFA4 = mergedCFA1;



for i = 1:1:numberOfCalls(1,1)
    mergedCFA1X = [];
    mergedCFA2X = mergedCFA1X;
    mergedCFA3X = mergedCFA1X;
    mergedCFA4X = mergedCFA1X;
    for j = 1:1:numberOfCalls(1,2)
        mergedCFA1X = cat(2,mergedCFA1X,trimmedCFA1(:,:,i,j));
        mergedCFA2X = cat(2,mergedCFA2X,trimmedCFA2(:,:,i,j));
        mergedCFA3X = cat(2,mergedCFA3X,trimmedCFA3(:,:,i,j));
        mergedCFA4X = cat(2,mergedCFA4X,trimmedCFA4(:,:,i,j));
    end
        mergedCFA1 = cat(1,mergedCFA1,mergedCFA1X);
        mergedCFA2 = cat(1,mergedCFA2,mergedCFA2X);
        mergedCFA3 = cat(1,mergedCFA3,mergedCFA3X);
        mergedCFA4 = cat(1,mergedCFA4,mergedCFA4X);
    
end


%% remove image mean for each mergedCFA
% This particular algorithm  was superseeded on the 28th of July 2018
% DCRemovedCFA1= mergedCFA1 - mean(mean(mergedCFA1));
% DCRemovedCFA2= mergedCFA2 - mean(mean(mergedCFA2));
% DCRemovedCFA3= mergedCFA3 - mean(mean(mergedCFA3));
% DCRemovedCFA4= mergedCFA4 - mean(mean(mergedCFA4));


%% create sparse noiseless arrays
% from 4 CFA array's create 3 colour specific arrays
%variables space save
patternNoiseRed = zeros(2.*(numberOfCalls(1,1)*(blockSize(1,1)-advance(1,1))),2.*(numberOfCalls(1,1)*(blockSize(1,1)-advance(1,1))));
patternNoiseGreen = patternNoiseRed;
patternNoiseBlue = patternNoiseRed;

patternNoiseRed(1:2:end,1:2:end) = mergedCFA1;
patternNoiseGreen(1:2:end,2:2:end) = mergedCFA2;
patternNoiseGreen(2:2:end,1:2:end) = mergedCFA3;
patternNoiseBlue(2:2:end,2:2:end) = mergedCFA4;

denoisedImage=cat(3,patternNoiseRed,patternNoiseGreen,patternNoiseBlue);


%% consturct refrence pattern to parse back
%perform crop to picture
shiftCFA1    =circshift(CFA1,-0.5.*advance(1,1),1);
shiftCFA1    =circshift(shiftCFA1,-0.5.*advance(1,2),2);

shiftCFA2    =circshift(CFA2,-0.5.*advance(1,1),1);
shiftCFA2    =circshift(shiftCFA2,-0.5.*advance(1,2),2);

shiftCFA3    =circshift(CFA3,-0.5.*advance(1,1),1);
shiftCFA3    =circshift(shiftCFA3,-0.5.*advance(1,2),2);

shiftCFA4    =circshift(CFA4,-0.5.*advance(1,1),1);
shiftCFA4    =circshift(shiftCFA4,-0.5.*advance(1,2),2);


% [a,b,c,d] = size(patternNoiseCFA1);
%remove end elements
% trimmedCFA1 = zeros(a-advance(1,1),b-advance(1,2),c,d);
% trimmedCFA2 = trimmedCFA1;
% trimmedCFA3 = trimmedCFA1;
% trimmedCFA4 = trimmedCFA1;

trimmedCFA1 = shiftCFA1(1:1:end-advance(1,1),1:1:end-advance(1,2));
trimmedCFA2 = shiftCFA2(1:1:end-advance(1,1),1:1:end-advance(1,2));
trimmedCFA3 = shiftCFA3(1:1:end-advance(1,1),1:1:end-advance(1,2));
trimmedCFA4 = shiftCFA4(1:1:end-advance(1,1),1:1:end-advance(1,2));

% % Remove Image DC
% 
% DCRemovedCFA1= trimmedCFA1 - mean(mean(trimmedCFA1));
% DCRemovedCFA2= trimmedCFA2 - mean(mean(trimmedCFA2));
% DCRemovedCFA3= trimmedCFA3 - mean(mean(trimmedCFA3));
% DCRemovedCFA4= trimmedCFA4 - mean(mean(trimmedCFA4));

%FFT method
% FTCFA1 = fft2(trimmedCFA1);
% FTCFA1(1,1) = 0;
% DCRemovedCFA1=ifft2(FTCFA1);
% 
% FTCFA2 = fft2(trimmedCFA2);
% FTCFA2(1,1) = 0;
% DCRemovedCFA2=ifft2(FTCFA2);
% 
% FTCFA3 = fft2(trimmedCFA3);
% FTCFA3(1,1) = 0;
% DCRemovedCFA3=ifft2(FTCFA3);
% 
% FTCFA4 = fft2(trimmedCFA4);
% FTCFA4(1,1) = 0;
% DCRemovedCFA4=ifft2(FTCFA4);

Red = zeros(2.*(numberOfCalls(1,1)*(blockSize(1,1)-advance(1,1))),2.*(numberOfCalls(1,1)*(blockSize(1,1)-advance(1,1))));
Green = Red;
Blue = Red;

% create sparse image in typical file format
%Red(1:2:end,1:2:end) = DCRemovedCFA1;
%Green(1:2:end,2:2:end) = DCRemovedCFA2;
%Green(2:2:end,1:2:end) = DCRemovedCFA3;
%Blue(2:2:end,2:2:end) = DCRemovedCFA4;

Red(1:2:end,1:2:end)= trimmedCFA1;
Green(1:2:end,2:2:end) = trimmedCFA2;
Green(2:2:end,1:2:end) = trimmedCFA3;
Blue(2:2:end,2:2:end) = trimmedCFA4;
% perform k = Y-I
PICTURE = cat(3,Red,Green,Blue);


PRNU = PICTURE-denoisedImage; % k=Y-I
%%Remove DC
PRNU = PRNU - mean(mean(mean(PRNU)));

PRNU = denoisedImage; %humour me.

% whos PRNU
% disp('mean');
% mean(mean(PRNU))
% disp('max');
% max(max(PRNU))
% disp('min');
% min(min(PRNU))

