function [PRNU,picture] = getRichardFP(file,crop,filter)

% mode one = fingerprint
% mode two = reference pattern

%Statics
if crop == 1
    crop = 1; % we will crop to center square 400x400
else
    clear crop; % remove variable and save memory
end


File_TIFF = '.tiff';
File_TIF = '.tif';
File_PEF = '.PEF';
File_BMP = '.bmp';
File_JPG = '.JPG';
File_jpg = '.jpg';
File_CANNON = '.CR2'; %#ok<NASGU>

% Set standard image size

% MUST BE EVEN NUMBER
cropSizeX = 900;
cropSizeY = 900;

%% read in image

% get fileName and fileExtension to conduct tests upon
[~,fileName,fileExtension]=fileparts(file);

% Pentax Image format
if strcmp(fileExtension,File_PEF)
    [picture] = readPentaxImage(file);  % 12bit signal
    [x,y,z] = size(picture);
     
    if crop == 1
        picture = picture(round(x/2)-(cropSizeX/2):round(x/2)+((cropSizeX/2)-1),round(y/2)-(cropSizeY/2):round(y/2+((cropSizeY/2)-1)),:); %centre crop image to cropSizeX x CropSizeY pixels
    end
    picture = int16(picture);

  %  fprintf('\t\t\t\t\tPentax');
elseif strcmp(fileExtension,File_TIFF)
    [picture] = readTiffImage(file);  % 12bit signal
    [x,y,z] = size(picture);
     
    if crop==1
        picture = picture(round(x/2)-(cropSizeX/2):round(x/2)+((cropSizeX/2)-1),round(y/2)-(cropSizeY/2):round(y/2+((cropSizeY/2)-1)),:); %centre crop image to cropSizeX x CropSizeY pixels
    end    
    % picture = double255(picture);% scale to 0-255
   % picture = picture.*64; %shift left 6 %shift from 10 to 16 
    picture = int16(picture);
    % fprintf('\t\t\t\t\tTiff');
elseif strcmp(fileExtension,File_BMP)
    
elseif strcmp(fileExtension,File_JPG)
        [picture] = imread(file); %8-bit depth
        [x,y,z] = size(picture);
     
        if crop == 1
            picture = picture(round(x/2)-(cropSizeX/2):round(x/2)+((cropSizeX/2)-1),round(y/2)-(cropSizeY/2):round(y/2+((cropSizeY/2)-1)),:); %centre crop image to cropSizeX x CropSizeY pixels
        end

elseif strcmp(fileExtension,File_jpg)
        [picture] = imread(file); %8-bit depth
        [x,y,z] = size(picture);
     
        if crop == 1
            picture = picture(round(x/2)-(cropSizeX/2):round(x/2)+((cropSizeX/2)-1),round(y/2)-(cropSizeY/2):round(y/2+((cropSizeY/2)-1)),:); %centre crop image to cropSizeX x CropSizeY pixels
        end
      
else
    fprintf('\nImage type not supported.\n') 
end

% convert  all signals to 8 bit signals

    switch class(picture);,            % convert to [0,255]
        case 'uint8',  picture=double(picture);
        case 'uint16', picture=ceil(double(picture)/4096*255);
        case 'int16',  picture=ceil(double(picture)/4096*255);
    end

    

       
% Break into RGB arrays and convert to greyscale

% if crop ==1
%     CFA1 = zeros(cropSizeX/2,cropSizeY/2);
%     CFA2 = zeros(cropSizeX/2,cropSizeY/2);
%     CFA3 = zeros(cropSizeX/2,cropSizeY/2);
%     CFA4 = zeros(cropSizeX/2,cropSizeY/2);
% else
%     CFA1 = zeros(x/2,y/2);
%     CFA2 = zeros(x/2,y/2);
%     CFA3 = zeros(x/2,y/2);
%     CFA4 = zeros(x/2,y/2);
% end


if z==3
     % ASSUMPTION CFA = (RG) (GB) colour quadrature
     CFA1 = picture(1:2:end,1:2:end,1);
     CFA2 = picture(1:2:end,2:2:end,2);
     CFA3 = picture(2:2:end,1:2:end,2);
     CFA4 = picture(2:2:end,2:2:end,3);
     
     %picture = 0.3.*R(:,:) + 0.6.*G(:,:) + 0.1.*B(:,:); %Matt Algorithm source?
     
else
    % ASSUMPTION ALL 1 DIM IMAGES ARE INTERPOLATED.
     CFA1 = picture(1:2:end,1:2:end);
     CFA2 = picture(1:2:end,2:2:end);
     CFA3 = picture(2:2:end,1:2:end);
     CFA4 = picture(2:2:end,2:2:end);
     
     %picture = 0.3.*R(:,:) + 0.6.*G(:,:) + 0.1.*B(:,:); %Matt Algorithm source?
end

% %% Filter White Noise Sources
if(filter==1)
 CFA1 = wiener2(CFA1,[8 8]);
 CFA2 = wiener2(CFA2,[8 8]);
 CFA3 = wiener2(CFA3,[8 8]);
 CFA4 = wiener2(CFA4,[8 8]);
end


%break image into four passes CFA1->4
%% zero mean image and std deviation of 1
    CFA1=CFA1-mean(mean(CFA1));
    CFA2=CFA2-mean(mean(CFA2));
    CFA3=CFA3-mean(mean(CFA3));
    CFA4=CFA4-mean(mean(CFA4));

    CFA1=CFA1/std(std(double(CFA1)));
    CFA2=CFA2/std(std(double(CFA2)));
    CFA3=CFA3/std(std(double(CFA3)));
    CFA4=CFA4/std(std(double(CFA4)));

% %% mask saturated pixels
% 
    CFA1(CFA1>(2^8-0.05*2^8))=0;
    CFA2(CFA2>(2^8-0.05*2^8))=0;
    CFA3(CFA3>(2^8-0.05*2^8))=0;
    CFA4(CFA4>(2^8-0.05*2^8))=0;


%% create filter
% 
% if crop ==1
% 	H = ones(cropSizeX/2,cropSizeY/2);
% else 
%     H = ones(x/2,y/2);
% end
% centreX = (cropSizeX/2)/2+1;
% centreY = (cropSizeY/2)/2+1;
% % From visual inspection of multiple images majority of the HF components
% % are limited to the following zones about the center.
% % - middle 9 pixels 
% % - lines N,S,E,W 50% of hight and width.
%     H((centreX),(centreY)) = 1;         % C
%     H((centreX)-1,(centreY)-1) = 0.25;    % NW
%     H((centreX)-1,(centreY)+1) = 0.25;    % NE
%     H((centreX)+1,(centreY)-1) = 0.25;    % SW
%     H((centreX)+1,(centreY)+1) = 0.25;    % SE
%     %north west
%     H((centreX)-2,(centreY)-2)=0.5;
%     H((centreX)-2,(centreY)-1)=0.5;
%     H((centreX)-1,(centreY)-2)=0.5;
%     
%     %north east
%     H((centreX)-2,(centreY)+2)=0.5;
%     H((centreX)-1,(centreY)+2)=0.5;
%     H((centreX)-2,(centreY)+1)=0.5;
%     
%     %south west
%     H((centreX)+2,(centreY)-2)=0.5;
%     H((centreX)+2,(centreY)-1)=0.5;
%     H((centreX)+1,(centreY)-2)=0.5;
%     
%     %south east
%     H((centreX)+2,(centreY)+2)=0.5;
%     H((centreX)+2,(centreY)+1)=0.5;
%     H((centreX)+1,(centreY)+2)=0.5;





%% Guassian filter
% D0= (150/1136)*pi; % Lukas et al ... need to validate why this cut off frequency was chosen.
D0 = 0.05*(x/2);
if crop == 1
    H = hpffilter('gaussian', cropSizeX/2, cropSizeY/2, D0);
else
    H = hpffilter('gaussian', x/2, y/2, D0);
end


%  guassian = [1 4 7 4 1; 4 16 28 16 4; 7 26 41 26 7; 4 16 28 16 4; 1 4 7 4 1];
%  HPF_guassian =normdata2(guassian);
% 
% H((centreX-2):(centreX-2)+4,(centreY-2):(centreY-2)+4)=HPF_guassian(:,:);
    
%%
PQ = paddedsize(size(CFA1));
[X,Y]=size(CFA1);
%D0 = 0.05*PQ(1);
D0 = (150/1136)*pi;
H = hpffilter('gaussian', PQ(1), PQ(2), D0); % Calculate the HPF



%% pass image to fourier domain


F1 = dct2(double(CFA1),size(H,1),size(H,2)); %use DCT instead of FFT so only real numbers are given
F2 = dct2(double(CFA2),size(H,1),size(H,2));
F3 = dct2(double(CFA3),size(H,1),size(H,2));
F4 = dct2(double(CFA4),size(H,1),size(H,2));
%F2 = fftshift(F); % shifted but not scaled                          }
                                                                    %} Mathematically unsure which one should be used for the reference pattern... does it matter?
%S2=fftshift(log(1+abs(F))); % shift and scale for visual comparison }

%% multiply filter

%H=(fftshift(H));

PRNU1 = idct2(H.*(F1));
PRNU2 = idct2(H.*(F2));
PRNU3 = idct2(H.*(F3));
PRNU4 = idct2(H.*(F4));

PRNU1=PRNU1(1:size(CFA1,1), 1:size(CFA1,2));
PRNU2=PRNU2(1:size(CFA2,1), 1:size(CFA2,2));
PRNU3=PRNU3(1:size(CFA3,1), 1:size(CFA3,2));
PRNU4=PRNU4(1:size(CFA4,1), 1:size(CFA4,2));


if crop ==1    
    PRNU = zeros(cropSizeX,cropSizeY);
else
    PRNU = zeros(x,y);
end
PRNU(1:2:end,1:2:end) = PRNU1;
PRNU(1:2:end,2:2:end) = PRNU2;
PRNU(2:2:end,1:2:end) = PRNU3;
PRNU(2:2:end,2:2:end) = PRNU4;

%[X,Y]=size(PRNU);

%PRNU = wiener2(PRNU ,[2 2]);
%I = Y-YK

% crop outer edge effects
%fingerprint = fingerprint(1:3,:); % frequency compoents are shifted to the outer coloumn of the image

