% function [picture, bit12] = readCannon2Image(imageFileName)
function [picture] = readCannon2Image(imageFileName)
% Read in a RAW PEF image file from a *ist DL Pentax Camera and return the
% relevant 3040 x 2024 raw image sensor 12-bit data output. Landscape
% images are assumed.
%Input:
%   imageFileName:      file name of image
%
%Output:
%   picture:            3040x2024 array of raw light in 12 bit 
%
% EXAMPLE
%
%   myPicture = readPentaxImage('IMGP7741.PEF')

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Function [picture, bit12] = readPentaxImage(imageFileName)
%
%   Written By Richard MATTHEWS
%
%   Based on Code originally written 20150317
%   Transferred to function call 20150330
%
%   Mr MATTHEWS is an academic collaborator at:
%   SCHOOL OF ELECTRICAL AND ELECTRONIC ENGINEERING
%   THE UNIVERSITY OF ADELAIDE
%   SA, AUSTRALIA 5005 
%   
%   This work was done as preliminary work for the award of MPhil in part
%   sponsered by the Australian Governemtn Department of Veterans' Affairs
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Exceptions
% if file is not.PEF throw exception?
% if file does not identify as PEF throw exception. --> This should be in
% TIFF header.

%% Constants
% these constants are hard coded but, given that the image is a TIFF file
% future carnations of this code should in fact read in the EXIF data of
% the file and then reconstruct the file based on the strip or tile data
% from the TIFF standard.

IMAGESTART = single(73728); %hypothesised image start HEX address
IMAGEEND = single(6152960); %hypothesised image end HEX address

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Richard Note 25th of May 2016. 
%
% These should really be replaced with a method call that finds the start
% of the image in accordance with the TIFF standard given that this is a
% PSEUDO TIFF file format regardless. For now this works however, may be an
% area of concern should the code start returning images that are not
% aligned correclty.
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%% Read in image file as text data file in HEX
fileID = fopen(imageFileName,'r'); %open file <name> for -reading
fseek(fileID,IMAGESTART,-1);
data = fread(fileID, IMAGEEND,'ubit12',0,'b'); % Saves 12-bits.... read in unsigined 12bits per number
fclose(fileID); %close file

% Bit shift to 16-bit samples to be consistant with TIFF and other images
%data = bitshift(data,-4);

%% reshape to image hxw
 picture = reshape(data,3040,2024);
 picture = uint16(picture');

